@@ -1,7 +1,7 @@
 /*
 
     MPDM - Minimum Profit Data Manager
-    Copyright (C) 2003/2007 Angel Ortega <angel@triptico.com>
+    Copyright (C) 2003/2009 Angel Ortega <angel@triptico.com>
 
     mpdm_f.c - File management
 
@@ -77,8 +77,8 @@ struct mpdm_file {
 	FILE *in;
 	FILE *out;
 
-	wchar_t * (* f_read) (const struct mpdm_file *, int *);
-	int	  (* f_write)(const struct mpdm_file *, const wchar_t *);
+	wchar_t * (* f_read) (struct mpdm_file *, int *);
+	int	  (* f_write)(struct mpdm_file *, const wchar_t *);
 
 #ifdef CONFOPT_ICONV
 	iconv_t ic_enc;
@@ -107,7 +107,7 @@ static void store_syserr(void)
 }
 
 
-static int get_char(const struct mpdm_file *f)
+static int get_char(struct mpdm_file *f)
 /* reads a character from a file structure */
 {
 	int c = EOF;
@@ -134,7 +134,7 @@ static int get_char(const struct mpdm_file *f)
 }
 
 
-static int put_buf(const char *ptr, int s, const struct mpdm_file *f)
+static int put_buf(const char *ptr, int s, struct mpdm_file *f)
 /* writes s bytes in the buffer in ptr to f */
 {
 #ifdef CONFOPT_WIN32
@@ -155,7 +155,7 @@ static int put_buf(const char *ptr, int s, const struct mpdm_file *f)
 }
 
 
-static int put_char(int c, const struct mpdm_file *f)
+static int put_char(int c, struct mpdm_file *f)
 /* writes a character in a file structure */
 {
 	char tmp = c;
@@ -167,7 +167,7 @@ static int put_char(int c, const struct mpdm_file *f)
 }
 
 
-static wchar_t *read_mbs(const struct mpdm_file *f, int *s)
+static wchar_t *read_mbs(struct mpdm_file *f, int *s)
 /* reads a multibyte string from a mpdm_file into a dynamic string */
 {
 	wchar_t *ptr = NULL;
@@ -213,7 +213,7 @@ static wchar_t *read_mbs(const struct mpdm_file *f, int *s)
 }
 
 
-static int write_wcs(const struct mpdm_file *f, const wchar_t * str)
+static int write_wcs(struct mpdm_file *f, const wchar_t * str)
 /* writes a wide string to an struct mpdm_file */
 {
 	int s;
@@ -229,7 +229,7 @@ static int write_wcs(const struct mpdm_file *f, const wchar_t * str)
 
 #ifdef CONFOPT_ICONV
 
-static wchar_t *read_iconv(const struct mpdm_file *f, int *s)
+static wchar_t *read_iconv(struct mpdm_file *f, int *s)
 /* reads a multibyte string transforming with iconv */
 {
 	char tmp[128];
@@ -286,7 +286,7 @@ static wchar_t *read_iconv(const struct mpdm_file *f, int *s)
 }
 
 
-static int write_iconv(const struct mpdm_file *f, const wchar_t * str)
+static int write_iconv(struct mpdm_file *f, const wchar_t * str)
 /* writes a wide string to a stream using iconv */
 {
 	char tmp[128];
@@ -333,8 +333,8 @@ static int write_iconv(const struct mpdm_file *f, const wchar_t * str)
 
 #define UTF8_BYTE() if((c = get_char(f)) == EOF) break
 
-static wchar_t *read_utf8(const struct mpdm_file *f, int *s)
-/* crappy, ad-hoc utf8 reader */
+static wchar_t *read_utf8(struct mpdm_file *f, int *s)
+/* utf8 reader */
 {
 	wchar_t *ptr = NULL;
 	wchar_t wc;
@@ -381,8 +381,8 @@ static wchar_t *read_utf8(const struct mpdm_file *f, int *s)
 }
 
 
-static int write_utf8(const struct mpdm_file *f, const wchar_t * str)
-/* crappy, ad-hoc utf8 writer */
+static int write_utf8(struct mpdm_file *f, const wchar_t * str)
+/* utf8 writer */
 {
 	int cnt = 0;
 	wchar_t wc;
@@ -411,8 +411,47 @@ static int write_utf8(const struct mpdm_file *f, const wchar_t * str)
 }
 
 
-static wchar_t *read_iso8859_1(const struct mpdm_file *f, int *s)
-/* crappy, ad-hoc iso8859-1 reader */
+static wchar_t *read_utf8_bom(struct mpdm_file *f, int *s)
+/* utf-8 reader with BOM detection */
+{
+	wchar_t *enc = L"";
+
+	f->f_read = NULL;
+
+	/* autodetection */
+	if (get_char(f) == 0xef && get_char(f) == 0xbb && get_char(f) == 0xbf)
+		enc = L"utf-8bom";
+	else {
+		enc = L"utf-8";
+		fseek(f->in, 0, SEEK_SET);
+	}
+
+	mpdm_hset_s(mpdm_root(), L"DETECTED_ENCODING", MPDM_LS(enc));
+
+	/* we're utf-8 from now on */
+	f->f_read = read_utf8;
+
+	return f->f_read(f, s);
+}
+
+
+static int write_utf8_bom(struct mpdm_file *f, const wchar_t * str)
+/* utf-8 writer with BOM */
+{
+	/* store the BOM */
+	put_char(0xef, f);
+	put_char(0xbb, f);
+	put_char(0xbf, f);
+
+	/* we're utf-8 from now on */
+	f->f_write = write_utf8;
+
+	return f->f_write(f, str);
+}
+
+
+static wchar_t *read_iso8859_1(struct mpdm_file *f, int *s)
+/* iso8859-1 reader */
 {
 	wchar_t *ptr = NULL;
 	wchar_t wc;
@@ -441,8 +480,8 @@ static wchar_t *read_iso8859_1(const struct mpdm_file *f, int *s)
 }
 
 
-static int write_iso8859_1(const struct mpdm_file *f, const wchar_t * str)
-/* crappy, ad-hoc iso8859-1 writer */
+static int write_iso8859_1(struct mpdm_file *f, const wchar_t * str)
+/* iso8859-1 writer */
 {
 	int cnt = 0;
 	wchar_t wc;
@@ -455,7 +494,7 @@ static int write_iso8859_1(const struct mpdm_file *f, const wchar_t * str)
 }
 
 
-static wchar_t *read_utf16ae(const struct mpdm_file *f, int *s, int le)
+static wchar_t *read_utf16ae(struct mpdm_file *f, int *s, int le)
 /* utf16 reader, ANY ending */
 {
 	wchar_t *ptr = NULL;
@@ -496,7 +535,7 @@ static wchar_t *read_utf16ae(const struct mpdm_file *f, int *s, int le)
 }
 
 
-static int write_utf16ae(const struct mpdm_file *f, const wchar_t * str, int le)
+static int write_utf16ae(struct mpdm_file *f, const wchar_t * str, int le)
 /* utf16 writer, ANY ending */
 {
 	int cnt = 0;
@@ -519,25 +558,25 @@ static int write_utf16ae(const struct mpdm_file *f, const wchar_t * str, int le)
 }
 
 
-static wchar_t *read_utf16le(const struct mpdm_file *f, int *s)
+static wchar_t *read_utf16le(struct mpdm_file *f, int *s)
 {
 	return read_utf16ae(f, s, 1);
 }
 
 
-static int write_utf16le(const struct mpdm_file *f, const wchar_t * str)
+static int write_utf16le(struct mpdm_file *f, const wchar_t * str)
 {
 	return write_utf16ae(f, str, 1);
 }
 
 
-static wchar_t *read_utf16be(const struct mpdm_file *f, int *s)
+static wchar_t *read_utf16be(struct mpdm_file *f, int *s)
 {
 	return read_utf16ae(f, s, 0);
 }
 
 
-static int write_utf16be(const struct mpdm_file *f, const wchar_t * str)
+static int write_utf16be(struct mpdm_file *f, const wchar_t * str)
 {
 	return write_utf16ae(f, str, 0);
 }
@@ -546,26 +585,32 @@ static int write_utf16be(const struct mpdm_file *f, const wchar_t * str)
 static wchar_t *read_utf16(struct mpdm_file *f, int *s)
 {
 	int c1, c2;
+	wchar_t *enc = L"utf-16le";
 
-	f->f_read = NULL;
+	/* assume little-endian */
+	f->f_read = read_utf16le;
 
 	/* autodetection */
 	c1 = get_char(f);
 	c2 = get_char(f);
 
-	if (c1 == 0xff && c2 == 0xfe)
-		f->f_read = read_utf16le;
-	else
-	if (c1 == 0xfe && c2 == 0xff)
+	if (c1 == 0xfe && c2 == 0xff) {
+		enc = L"utf-16be";
 		f->f_read = read_utf16be;
+	}
 	else
-		return NULL;
+	if (c1 != 0xff || c2 != 0xfe) {
+		/* no BOM; rewind and hope */
+		fseek(f->in, 0, SEEK_SET);
+	}
+
+	mpdm_hset_s(mpdm_root(), L"DETECTED_ENCODING", MPDM_LS(enc));
 
 	return f->f_read(f, s);
 }
 
 
-static int write_utf16(struct mpdm_file *f, const wchar_t * str)
+static int write_utf16le_bom(struct mpdm_file *f, const wchar_t * str)
 {
 	/* store the LE signature */
 	put_char(0xff, f);
@@ -578,7 +623,20 @@ static int write_utf16(struct mpdm_file *f, const wchar_t * str)
 }
 
 
-static wchar_t *read_utf32ae(const struct mpdm_file *f, int *s, int le)
+static int write_utf16be_bom(struct mpdm_file *f, const wchar_t * str)
+{
+	/* store the BE signature */
+	put_char(0xfe, f);
+	put_char(0xff, f);
+
+	/* we're 16be from now on */
+	f->f_write = write_utf16be;
+
+	return f->f_write(f, str);
+}
+
+
+static wchar_t *read_utf32ae(struct mpdm_file *f, int *s, int le)
 /* utf32 reader, ANY ending */
 {
 	wchar_t *ptr = NULL;
@@ -625,7 +683,7 @@ static wchar_t *read_utf32ae(const struct mpdm_file *f, int *s, int le)
 }
 
 
-static int write_utf32ae(const struct mpdm_file *f, const wchar_t * str, int le)
+static int write_utf32ae(struct mpdm_file *f, const wchar_t * str, int le)
 /* utf32 writer, ANY ending */
 {
 	int cnt = 0;
@@ -652,25 +710,25 @@ static int write_utf32ae(const struct mpdm_file *f, const wchar_t * str, int le)
 }
 
 
-static wchar_t *read_utf32le(const struct mpdm_file *f, int *s)
+static wchar_t *read_utf32le(struct mpdm_file *f, int *s)
 {
 	return read_utf32ae(f, s, 1);
 }
 
 
-static int write_utf32le(const struct mpdm_file *f, const wchar_t * str)
+static int write_utf32le(struct mpdm_file *f, const wchar_t * str)
 {
 	return write_utf32ae(f, str, 1);
 }
 
 
-static wchar_t *read_utf32be(const struct mpdm_file *f, int *s)
+static wchar_t *read_utf32be(struct mpdm_file *f, int *s)
 {
 	return read_utf32ae(f, s, 0);
 }
 
 
-static int write_utf32be(const struct mpdm_file *f, const wchar_t * str)
+static int write_utf32be(struct mpdm_file *f, const wchar_t * str)
 {
 	return write_utf32ae(f, str, 0);
 }
@@ -679,8 +737,9 @@ static int write_utf32be(const struct mpdm_file *f, const wchar_t * str)
 static wchar_t *read_utf32(struct mpdm_file *f, int *s)
 {
 	int c1, c2, c3, c4;
+	wchar_t *enc = L"utf-32le";
 
-	f->f_read = NULL;
+	f->f_read = read_utf32le;
 
 	/* autodetection */
 	c1 = get_char(f);
@@ -688,19 +747,22 @@ static wchar_t *read_utf32(struct mpdm_file *f, int *s)
 	c3 = get_char(f);
 	c4 = get_char(f);
 
-	if (c1 == 0xff && c2 == 0xfe && c3 == 0 && c4 == 0)
-		f->f_read = read_utf32le;
-	else
-	if (c1 == 0 && c2 == 0 && c3 == 0xfe && c4 == 0xff)
+	if (c1 == 0 && c2 == 0 && c3 == 0xfe && c4 == 0xff) {
+		enc = L"utf-32be";
 		f->f_read = read_utf32be;
-	else
-		return NULL;
+	}
+	if (c1 != 0xff || c2 != 0xfe || c3 != 0 || c4 != 0) {
+		/* no BOM; assume le and hope */
+		fseek(f->in, 0, SEEK_SET);
+	}
+
+	mpdm_hset_s(mpdm_root(), L"DETECTED_ENCODING", MPDM_LS(enc));
 
 	return f->f_read(f, s);
 }
 
 
-static int write_utf32(struct mpdm_file *f, const wchar_t * str)
+static int write_utf32le_bom(struct mpdm_file *f, const wchar_t * str)
 {
 	/* store the LE signature */
 	put_char(0xff, f);
@@ -715,6 +777,132 @@ static int write_utf32(struct mpdm_file *f, const wchar_t * str)
 }
 
 
+static int write_utf32be_bom(struct mpdm_file *f, const wchar_t * str)
+{
+	/* store the BE signature */
+	put_char(0, f);
+	put_char(0, f);
+	put_char(0xfe, f);
+	put_char(0xff, f);
+
+	/* we're 32be from now on */
+	f->f_write = write_utf32be;
+
+	return f->f_write(f, str);
+}
+
+
+static wchar_t *read_auto(struct mpdm_file *f, int *s)
+/* autodetects different encodings based on the BOM */
+{
+	wchar_t *enc = L"";
+
+	/* by default, multibyte reading */
+	f->f_read = read_mbs;
+
+	/* ensure seeking is possible */
+	if (f->in != NULL && fseek(f->in, 0, SEEK_CUR) != -1) {
+		int c;
+
+		c = get_char(f);
+
+		if (c == 0xff) {
+			/* can be utf32le or utf16le */
+			if (get_char(f) == 0xfe) {
+				/* if next 2 chars are 0x00, it's utf32; otherwise utf16 */
+				if (get_char(f) == 0x00 && get_char(f) == 0x00) {
+					enc = L"utf-32le";
+					f->f_read = read_utf32le;
+					goto got_encoding;
+				}
+				else {
+					/* rewind to 3rd character */
+					fseek(f->in, 2, SEEK_SET);
+
+					enc = L"utf-16le";
+					f->f_read = read_utf16le;
+					goto got_encoding;
+				}
+			}
+		}
+		else
+		if (c == 0x00) {
+			/* can be utf32be */
+			if (get_char(f) == 0x00 && get_char(f) == 0xfe && get_char(f) == 0xff) {
+				enc = L"utf-32be";
+				f->f_read = read_utf32be;
+				goto got_encoding;
+			}
+		}
+		else
+		if (c == 0xfe) {
+			/* can be utf16be */
+			if (get_char(f) == 0xff) {
+				enc = L"utf-16be";
+				f->f_read = read_utf16be;
+				goto got_encoding;
+			}
+		}
+		else
+		if (c == 0xef) {
+			/* can be utf8 with BOM */
+			if (get_char(f) == 0xbb && get_char(f) == 0xbf) {
+				enc = L"utf-8bom";
+				f->f_read = read_utf8;
+				goto got_encoding;
+			}
+		}
+		else {
+			/* try if a first bunch of chars are valid UTF-8 */
+			int p = c;
+			int n = 10000;
+			int u = 0;
+
+			while (--n && (c = get_char(f)) != EOF) {
+				if ((c & 0xc0) == 0x80) {
+					if ((p & 0xc0) == 0xc0)
+						u++;
+					else
+					if ((p & 0x80) == 0x00) {
+						u = -1;
+						break;
+					}
+				}
+				else
+				if ((p & 0xc0) == 0xc0) {
+					u = -1;
+					break;
+				}
+
+				p = c;
+			}
+
+			if (u < 0) {
+				/* invalid utf-8; fall back to 8bit */
+				enc = L"8bit";
+				f->f_read = read_iso8859_1;
+			}
+			else
+			if (u > 0) {
+				/* utf-8 sequences found */
+				enc = L"utf-8";
+				f->f_read = read_utf8;
+			}
+
+			/* 7 bit ASCII: do nothing */
+		}
+
+		/* none of the above; restart */
+		fseek(f->in, 0, SEEK_SET);
+	}
+
+got_encoding:
+	mpdm_hset_s(mpdm_root(), L"DETECTED_ENCODING", MPDM_LS(enc));
+
+	return f->f_read(f, s);
+}
+
+
 static mpdm_t new_mpdm_file(void)
 /* creates a new file value */
 {
@@ -728,7 +916,7 @@ static mpdm_t new_mpdm_file(void)
 	memset(fs, '\0', sizeof(struct mpdm_file));
 
 	/* default I/O functions */
-	fs->f_read = read_mbs;
+	fs->f_read = read_auto;
 	fs->f_write = write_wcs;
 
 #ifdef CONFOPT_ICONV
@@ -741,62 +929,74 @@ static mpdm_t new_mpdm_file(void)
 		return NULL;
 	}
 
-	if ((e = mpdm_hget_s(mpdm_root(), L"ENCODING")) != NULL) {
+	e = mpdm_hget_s(mpdm_root(), L"ENCODING");
+
+	if (mpdm_size(e) == 0)
+		e = mpdm_hget_s(mpdm_root(), L"TEMP_ENCODING");
+
+	if (mpdm_size(e)) {
 
 		wchar_t *enc = mpdm_string(e);
 
-#ifdef CONFOPT_ICONV
-		mpdm_t cs = MPDM_2MBS(e->data);
-
-		if ((fs->ic_enc = iconv_open((char *) cs->data, "WCHAR_T")) != (iconv_t) -1 &&
-		    (fs->ic_dec = iconv_open("WCHAR_T", (char *) cs->data)) != (iconv_t) -1) {
-
-			fs->f_read = read_iconv;
-			fs->f_write = write_iconv;
-
-			return v;
-		}
-#endif				/* CONFOPT_ICONV */
-
 		if (wcscmp(enc, L"utf-8") == 0) {
-			fs->f_read = read_utf8;
+			fs->f_read = read_utf8_bom;
 			fs->f_write = write_utf8;
 		}
 		else
-		if (wcscmp(enc, L"iso8859-1") == 0) {
+		if (wcscmp(enc, L"utf-8bom") == 0) {
+			fs->f_read = read_utf8_bom;
+			fs->f_write = write_utf8_bom;
+		}
+		else
+		if (wcscmp(enc, L"iso8859-1") == 0 ||
+			wcscmp(enc, L"8bit") == 0) {
 			fs->f_read = read_iso8859_1;
 			fs->f_write = write_iso8859_1;
 		}
 		else
 		if (wcscmp(enc, L"utf-16le") == 0) {
 			fs->f_read = read_utf16le;
-			fs->f_write = write_utf16le;
+			fs->f_write = write_utf16le_bom;
 		}
 		else
 		if (wcscmp(enc, L"utf-16be") == 0) {
 			fs->f_read = read_utf16be;
-			fs->f_write = write_utf16be;
+			fs->f_write = write_utf16be_bom;
 		}
 		else
 		if (wcscmp(enc, L"utf-16") == 0) {
 			fs->f_read = read_utf16;
-			fs->f_write = write_utf16;
+			fs->f_write = write_utf16le_bom;
 		}
 		else
 		if (wcscmp(enc, L"utf-32le") == 0) {
 			fs->f_read = read_utf32le;
-			fs->f_write = write_utf32le;
+			fs->f_write = write_utf32le_bom;
 		}
 		else
 		if (wcscmp(enc, L"utf-32be") == 0) {
 			fs->f_read = read_utf32be;
-			fs->f_write = write_utf32be;
+			fs->f_write = write_utf32be_bom;
 		}
 		else
 		if (wcscmp(enc, L"utf-32") == 0) {
 			fs->f_read = read_utf32;
-			fs->f_write = write_utf32;
+			fs->f_write = write_utf32le_bom;
 		}
+		else {
+#ifdef CONFOPT_ICONV
+			mpdm_t cs = MPDM_2MBS(e->data);
+
+			if ((fs->ic_enc = iconv_open((char *) cs->data, "WCHAR_T")) != (iconv_t) -1 &&
+			    (fs->ic_dec = iconv_open("WCHAR_T", (char *) cs->data)) != (iconv_t) -1) {
+
+				fs->f_read = read_iconv;
+				fs->f_write = write_iconv;
+			}
+#endif				/* CONFOPT_ICONV */
+		}
+
+		mpdm_hset_s(mpdm_root(), L"TEMP_ENCODING", NULL);
 	}
 
 	return v;
@@ -880,6 +1080,15 @@ mpdm_t mpdm_new_f(FILE * f)
  * Opens a file. If @filename can be open in the specified @mode, an
  * mpdm_t value will be returned containing the file descriptor, or NULL
  * otherwise.
+ *
+ * If the file is open for reading, some charset detection methods are
+ * used. If any of them is successful, its name is stored in the
+ * DETECTED_ENCODING element of the mpdm_root() hash. This value is
+ * suitable to be copied over ENCODING or TEMP_ENCODING.
+ *
+ * If the file is open for writing, the encoding to be used is read from
+ * the ENCODING element of mpdm_root() and, if not set, from the
+ * TEMP_ENCODING one. The latter will always be deleted afterwards.
  * [File Management]
  */
 mpdm_t mpdm_open(const mpdm_t filename, const mpdm_t mode)
@@ -954,7 +1163,7 @@ mpdm_t mpdm_read(const mpdm_t fd)
 	mpdm_t v = NULL;
 	wchar_t *ptr;
 	int s;
-	const struct mpdm_file *fs = fd->data;
+	struct mpdm_file *fs = (struct mpdm_file *)fd->data;
 
 	if (fs == NULL)
 		return NULL;
@@ -972,7 +1181,7 @@ mpdm_t mpdm_getchar(const mpdm_t fd)
 {
 	int c;
 	wchar_t tmp[2];
-	const struct mpdm_file *fs = fd->data;
+	struct mpdm_file *fs = (struct mpdm_file *)fd->data;
 
 	if (fs == NULL || (c = get_char(fs)) == EOF)
 		return NULL;
@@ -987,7 +1196,7 @@ mpdm_t mpdm_getchar(const mpdm_t fd)
 
 mpdm_t mpdm_putchar(const mpdm_t fd, const mpdm_t c)
 {
-	const struct mpdm_file *fs = fd->data;
+	struct mpdm_file *fs = (struct mpdm_file *)fd->data;
 	const wchar_t *ptr = mpdm_string(c);
 
 	if (fs == NULL || put_char(*ptr, fs) == -1)
@@ -1008,7 +1217,7 @@ mpdm_t mpdm_putchar(const mpdm_t fd, const mpdm_t c)
  */
 int mpdm_write(const mpdm_t fd, const mpdm_t v)
 {
-	const struct mpdm_file *fs = fd->data;
+	struct mpdm_file *fs = (struct mpdm_file *)fd->data;
 	int ret = -1;
 
 	if (fs == NULL)
@@ -1022,7 +1231,7 @@ int mpdm_write(const mpdm_t fd, const mpdm_t v)
 
 int mpdm_fseek(const mpdm_t fd, long offset, int whence)
 {
-	const struct mpdm_file *fs = fd->data;
+	struct mpdm_file *fs = (struct mpdm_file *)fd->data;
 
 	return fseek(fs->in, offset, whence);
 }
@@ -1030,7 +1239,7 @@ int mpdm_fseek(const mpdm_t fd, long offset, int whence)
 
 long mpdm_ftell(const mpdm_t fd)
 {
-	const struct mpdm_file *fs = fd->data;
+	struct mpdm_file *fs = (struct mpdm_file *)fd->data;
 
 	return ftell(fs->in);
 }
@@ -1041,7 +1250,7 @@ FILE * mpdm_get_filehandle(const mpdm_t fd)
 	FILE * f = NULL;
 
 	if (fd->flags & MPDM_FILE) {
-		const struct mpdm_file *fs = fd->data;
+		struct mpdm_file *fs = (struct mpdm_file *)fd->data;
 		f = fs->in;
 	}
 
@@ -1069,6 +1278,7 @@ static mpdm_t embedded_encodings(void)
 		L"utf8",	NULL,
 		L"iso8859-1",	L"iso8859-1",
 		L"iso-8859-1",	NULL,
+		L"8bit",	NULL,
 		L"latin1",	NULL,
 		L"latin-1",	NULL,
 		L"utf-16le",	L"utf-16le",
@@ -1091,6 +1301,8 @@ static mpdm_t embedded_encodings(void)
 		L"utf32",	NULL,
 		L"ucs-4",	NULL,
 		L"ucs4",	NULL,
+		L"utf-8bom",	L"utf-8bom",
+		L"utf8bom",	NULL,
 		NULL,		NULL
 	};
 
@@ -1127,6 +1339,10 @@ static mpdm_t embedded_encodings(void)
  * and converted on each read / write. If charset is NULL, it
  * is reverted to default charset conversion (i.e. the one defined
  * in the locale).
+ *
+ * This function stores the @charset value into the ENCODING item
+ * of the mpdm_root() hash.
+ *
  * Returns a negative number if @charset is unsupported, or zero
  * if no errors were found.
  * [File Management]
@@ -1139,7 +1355,7 @@ int mpdm_encoding(mpdm_t charset)
 	mpdm_t v = NULL;
 
 	/* NULL encoding? done */
-	if (charset == NULL) {
+	if (mpdm_size(charset) == 0) {
 		mpdm_hset_s(mpdm_root(), L"ENCODING", NULL);
 		return 0;
 	}
@@ -1558,7 +1774,7 @@ static int sysdep_popen(mpdm_t v, char *prg, int rw)
 
 static int sysdep_pclose(const mpdm_t v)
 {
-	const struct mpdm_file *fs = v->data;
+	struct mpdm_file *fs = v->data;
 
 	if (fs->hin != NULL)
 		CloseHandle(fs->hin);
@@ -1635,7 +1851,7 @@ static int sysdep_pclose(const mpdm_t v)
 /* unix-style pipe close */
 {
 	int s;
-	const struct mpdm_file *fs = v->data;
+	struct mpdm_file *fs = (struct mpdm_file *)v->data;
 
 	if (fs->in != NULL)
 		fclose(fs->in);
