@@ -751,6 +751,32 @@ static gint key_press_event(GtkWidget * widget, GdkEventKey * event, gpointer da
 		case GDK_F12:		ptr = L"ctrl-f12"; break;
 		case GDK_KP_Enter:
 		case GDK_Return:	ptr = L"ctrl-enter"; break;
+		case GDK_Cyrillic_ve:	ptr = L"ctrl-d"; break;
+		case GDK_Cyrillic_a:	ptr = L"ctrl-f"; break;
+		case GDK_Cyrillic_tse:	ptr = L"ctrl-w"; break;
+		case GDK_Cyrillic_de:	ptr = L"ctrl-l"; break;
+		case GDK_Cyrillic_ie:	ptr = L"ctrl-t"; break;
+		case GDK_Cyrillic_ef:	ptr = L"ctrl-a"; break;
+		case GDK_Cyrillic_ghe:	ptr = L"ctrl-u"; break;
+		case GDK_Cyrillic_i:	ptr = L"ctrl-b"; break;
+		case GDK_Cyrillic_shorti:	ptr = L"ctrl-q"; break;
+		case GDK_Cyrillic_ka:	ptr = L"ctrl-r"; break;
+		case GDK_Cyrillic_el:	ptr = L"ctrl-k"; break;
+		case GDK_Cyrillic_em:	ptr = L"ctrl-v"; break;
+		case GDK_Cyrillic_en:	ptr = L"ctrl-y"; break;
+		case GDK_Cyrillic_o:	ptr = L"ctrl-j"; break;
+		case GDK_Cyrillic_pe:	ptr = L"ctrl-g"; break;
+		case GDK_Cyrillic_ya:	ptr = L"ctrl-z"; break;
+		case GDK_Cyrillic_er:	ptr = L"ctrl-h"; break;
+		case GDK_Cyrillic_es:	ptr = L"ctrl-c"; break;
+		case GDK_Cyrillic_te:	ptr = L"ctrl-n"; break;
+		case GDK_Cyrillic_softsign:	ptr = L"ctrl-m"; break;
+		case GDK_Cyrillic_yeru:	ptr = L"ctrl-s"; break;
+		case GDK_Cyrillic_ze:	ptr = L"ctrl-p"; break;
+		case GDK_Cyrillic_sha:	ptr = L"ctrl-i"; break;
+		case GDK_Cyrillic_e:	ptr = L"ctrl-t"; break;
+		case GDK_Cyrillic_shcha:	ptr = L"ctrl-o"; break;
+		case GDK_Cyrillic_che:	ptr = L"ctrl-x"; break;
 		}
 
 		if (ptr == NULL) {
@@ -911,6 +937,14 @@ static gint motion_notify_event(GtkWidget *widget, GdkEventMotion * event, gpoin
 }
 
 
+static void drag_data_received(GtkWidget *widget, GdkDragContext *dc, gint x, gint y,
+				GtkSelectionData *data, guint info, guint time)
+/* 'drag_data_received' handler */
+{
+	printf("drag_data_received (unsupported)\n");
+}
+
+
 /** clipboard functions **/
 
 static void commit(GtkIMContext * i, char * str, gpointer u)
@@ -986,6 +1020,10 @@ static void selection_get(GtkWidget * widget,
 
 	/* gets the clipboard and joins */
 	d = mpdm_hget_s(mp, L"clipboard");
+
+	if (mpdm_size(d) == 0)
+		return;
+
 	d = mpdm_join(MPDM_LS(L"\n"), d);
 
 	/* convert to current locale */
@@ -1582,6 +1620,10 @@ static mpdm_t gtk_drv_startup(mpdm_t a)
 	GdkScreen *screen;
 	mpdm_t v;
 	int w, h;
+	GtkTargetEntry targets[] = {
+		{ "text/plain",		0,	0 },
+		{ "text/uri-list",	0,	1 }
+	};
 
 	register_functions();
 
@@ -1662,6 +1704,12 @@ static mpdm_t gtk_drv_startup(mpdm_t a)
 	g_signal_connect(G_OBJECT(area), "scroll_event",
 		G_CALLBACK(scroll_event), NULL);
 
+	gtk_drag_dest_set(area, GTK_DEST_DEFAULT_ALL, targets,
+		sizeof(targets) / sizeof(GtkTargetEntry),
+		GDK_ACTION_COPY);
+	g_signal_connect(G_OBJECT(area), "drag_data_received",
+		G_CALLBACK(drag_data_received), NULL);
+
 	gtk_selection_add_target(area, GDK_SELECTION_PRIMARY,
 		GDK_SELECTION_TYPE_STRING, 1);
 
