@@ -105,7 +105,7 @@ static void update_window_size(void)
 
 	/* calculate the size in chars */
 	tx = ((rect.right - rect.left) / font_width) + 1;
-	ty = ((rect.bottom - rect.top - tab_height) / font_height) + 1;
+	ty = (rect.bottom - rect.top - tab_height) / font_height;
 
 	/* store the 'window' size */
 	v = mpdm_hget_s(mp, L"window");
@@ -512,7 +512,6 @@ static void win32_vkey(int c)
 		case VK_NEXT:		ptr = L"page-down"; break;
 		case VK_HOME:		ptr = L"home"; break;
 		case VK_END:		ptr = L"end"; break;
-		case VK_TAB:		ptr = L"tab"; break;
 		case VK_RETURN: 	ptr = L"enter"; break;
 		case VK_BACK:		ptr = L"backspace"; break;
 		case VK_DELETE: 	ptr = L"delete"; break;
@@ -555,9 +554,6 @@ static void win32_akey(int k)
 	wchar_t c[2];
 	wchar_t * ptr = NULL;
 
-	if (is_wm_keydown)
-		return;
-
 	/* set mp.shift_pressed */
 	if (GetKeyState(VK_SHIFT) & 0x8000)
 		mpdm_hset_s(mp, L"shift_pressed", MPDM_I(1));
@@ -571,12 +567,12 @@ static void win32_akey(int k)
 	case ctrl('e'):		ptr = L"ctrl-e"; break;
 	case ctrl('f'):		ptr = L"ctrl-f"; break;
 	case ctrl('g'):		ptr = L"ctrl-g"; break;
-	case ctrl('h'):		ptr = L"ctrl-h"; break;
+	case ctrl('h'):		/* same as backspace */ break;
 	case ctrl('i'):		ptr = L"ctrl-i"; break;
 	case ctrl('j'):		ptr = L"ctrl-j"; break;
 	case ctrl('k'):		ptr = L"ctrl-k"; break;
 	case ctrl('l'):		ptr = L"ctrl-l"; break;
-	case ctrl('m'):		ptr = L"ctrl-m"; break;
+	case ctrl('m'):		/* same as ENTER */ break;
 	case ctrl('n'):		ptr = L"ctrl-n"; break;
 	case ctrl('o'):		ptr = L"ctrl-o"; break;
 	case ctrl('p'):		ptr = L"ctrl-p"; break;
@@ -656,6 +652,28 @@ static void action_by_menu(int item)
 }
 
 
+static void dropped_files(HDROP hDrop)
+/* fill the mp.dropped_files array with the dropped files */
+{
+	mpdm_t a = MPDM_A(0);
+	char tmp[1024];
+	int n;
+
+	n = DragQueryFile(hDrop, 0xffffffff, NULL, sizeof(tmp) - 1);
+
+	while (--n >= 0) {
+		DragQueryFile(hDrop, n, tmp, sizeof(tmp) - 1);
+		mpdm_push(a, MPDM_MBS(tmp));
+	}
+
+	DragFinish(hDrop);
+
+	mpdm_hset_s(mp, L"dropped_files", a);
+	mp_process_event(MPDM_LS(L"dropped-files"));
+	redraw();
+}
+
+
 #ifndef WM_MOUSEWHEEL
 #define WM_MOUSEWHEEL			0x020A
 #endif
@@ -674,11 +692,11 @@ long STDCALL WndProc(HWND hwnd, UINT msg, UINT wparam, LONG lparam)
 		DragAcceptFiles(hwnd, TRUE);
 		return 0;
 
-/*	case WM_DROPFILES:
+	case WM_DROPFILES:
+
+		dropped_files((HDROP) wparam);
+		return 0;
 
-		(void) load_dropped_files ((HANDLE) wparam, hwnd);
-		return(0);
-*/
 	case WM_KEYUP:
 
 		is_wm_keydown = 0;
@@ -831,7 +849,7 @@ long STDCALL WndProc(HWND hwnd, UINT msg, UINT wparam, LONG lparam)
 	if (mp_exit_requested)
 		PostMessage(hwnd, WM_CLOSE, 0, 0);
 
-	return DefWindowProc(hwnd, msg, wparam, lparam);
+	return DefWindowProcW(hwnd, msg, wparam, lparam);
 }
 
 
@@ -846,6 +864,10 @@ static mpdm_t win32_drv_clip_to_sys(mpdm_t a)
 
 	/* convert the clipboard to DOS text */
 	d = mpdm_hget_s(mp, L"clipboard");
+
+	if (mpdm_size(d) == 0)
+		return NULL;
+
 	d = mpdm_join(MPDM_LS(L"\r\n"), d);
 	ptr = mpdm_wcstombs(d->data, &s);
 
@@ -1447,7 +1469,7 @@ static void register_functions(void)
 
 static mpdm_t win32_drv_startup(mpdm_t a)
 {
-	WNDCLASS wc;
+	WNDCLASSW wc;
 	RECT r;
 	mpdm_t v;
 
@@ -1465,12 +1487,12 @@ static mpdm_t win32_drv_startup(mpdm_t a)
 	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
 	wc.hbrBackground = NULL;
 	wc.lpszMenuName = NULL;
-	wc.lpszClassName = "minimumprofit5.x";
+	wc.lpszClassName = L"minimumprofit5.x";
 
-	RegisterClass(&wc);
+	RegisterClassW(&wc);
 
 	/* create the window */
-	hwnd = CreateWindow("minimumprofit5.x", "mp " VERSION,
+	hwnd = CreateWindowW(L"minimumprofit5.x", L"mp " VERSION,
 		WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN | WS_VSCROLL,
 		CW_USEDEFAULT, CW_USEDEFAULT,
 		CW_USEDEFAULT, CW_USEDEFAULT,
